<?php
/**
 * Created by PhpStorm.
 * User: jderosa
 * Date: 21/04/20
 * Time: 12.02
 */

namespace FiloBlu\Rma\Handlers;
use FiloBlu\Rma\Helper\ChangeSizeAttributeHelper;
use FiloBlu\Rma\Api\RmaExtensionAttributesRepositoryInterface;
use Psr\Log\LoggerInterface;
use FiloBlu\Rma\Helper\ChangeSizeOrder;
use Magento\Framework\App\RequestInterface;


class CreateChangeSizeOrderHandler
{
    /**
     * @var ChangeSizeAttributeHelper
     */
    private $changeSizeAttributeHelper;
    /**
     * @var RmaExtensionAttributesRepositoryInterface
     */
    private $rmaExtensionAttributesRepository;

    /**
     * @var array
     */
    protected $changSizeItems = [];
    /**
     * @var LoggerInterface
     */
    private $logger;
    /**
     * @var ChangeSizeOrder
     */
    private $changeSizeOrder;
    /**
     * @var RequestInterface
     */
    private $request;

    /**
     * CreateChangeSizeOrderHandler constructor.
     * @param ChangeSizeAttributeHelper $changeSizeAttributeHelper
     * @param RmaExtensionAttributesRepositoryInterface $rmaExtensionAttributesRepository
     * @param LoggerInterface $logger
     * @param ChangeSizeOrder $changeSizeOrder
     * @param RequestInterface $request
     */
    public function __construct(
        ChangeSizeAttributeHelper $changeSizeAttributeHelper,
        RmaExtensionAttributesRepositoryInterface $rmaExtensionAttributesRepository,
        LoggerInterface $logger,
        ChangeSizeOrder $changeSizeOrder,
        RequestInterface $request){
        $this->changeSizeAttributeHelper = $changeSizeAttributeHelper;
        $this->rmaExtensionAttributesRepository = $rmaExtensionAttributesRepository;
        $this->logger = $logger;
        $this->changeSizeOrder = $changeSizeOrder;
        $this->request = $request;
    }


    /**
     * @param \Magento\Rma\Model\Rma $subject
     * @param $result
     * @return mixed
     */
    public function checkIfCreateChangeSizeOrder(\Magento\Rma\Model\Rma $subject, $result){
        try {

            if ($result && isset($result['items'])) {

                $rmaItems = $result['items'];
                $parentProduct = null;

                foreach ($rmaItems as $item) {
                    if ($item->getStatus() == \Magento\Rma\Model\Rma\Source\Status::STATE_PENDING && $item->getResolution() == $this->changeSizeAttributeHelper->getChangeSizeAttributeOptionId()) {

                        if (!$this->rmaExtensionAttributesRepository->getChangeSizeOrderId($item->getId())) {

                            // check if customer
                            if (isset($item->getData()[ChangeSizeAttributeHelper::CHANGE_SIZE_RMA_ATTRIBUTE_CODE])
                                && $item->getData()[ChangeSizeAttributeHelper::CHANGE_SIZE_RMA_ATTRIBUTE_CODE] != "") {
                                $changeProductId = $item->getData()[ChangeSizeAttributeHelper::CHANGE_SIZE_RMA_ATTRIBUTE_CODE];
                                $parentProduct = $item->getProductSku();
                                $this->changSizeItems = array(
                                    'product_id' => $changeProductId,
                                    'qty' => $item['qty_requested'],
                                    'parent_product_sku' => $item->getProductSku(),
                                    'rma_entity_id' => $item->getEntityId(),
                                    'original_order_item_id' => $item->getOrderItemId()
                                );
                            }
                            $customEmail = $subject->getCustomerCustomEmail();
                            if (isset($customEmail) && $customEmail != "") {
                                $subject->getOrder()->setCustomerEmail($customEmail);
                            }
                            // create an order for each item requested
                            $this->createChangeSizeOrder($subject->getOrder(), $this->changSizeItems, $subject->getEntityId(), $parentProduct);

                        }
                    }
                }
            }
        } catch (\Exception $e) {
            $this->logger->critical($e->getMessage());
        } catch (\Throwable $t) {
            $this->logger->critical($t->getMessage());
        }
        return $result;
    }


    /**
     *
     * @param \Magento\Sales\Model\Order $order
     * @param array $items
     * @param int $rmaId
     * @param string|null $parentProduct
     */
    protected function createChangeSizeOrder($order, array $items, $rmaId, $parentProduct = null)
    {

        $orderData = [
            'order' => $order,
            'items' => $items,
            'rma_id' => $rmaId,
            'parent_product_sku' => $parentProduct,
            'request' => $this->request->getParams()
        ];

        if ($parentProduct !== null && is_numeric($items['product_id'])){
            $this->changeSizeOrder->createMageOrder($orderData);
        }
    }

}